/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.content.Context;
import android.database.Cursor;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Data;
import android.telephony.PhoneNumberUtils;
import android.util.Log;

import java.util.HashMap;

/**
 * 该类用于从设备联系人中根据电话号码获取联系人姓名，使用缓存机制提高性能。
 */
public class Contact {
    // 静态的联系人缓存，用于存储已经查询过的电话号码及其对应的联系人姓名
    private static HashMap<String, String> sContactCache;
    // 日志标签，用于在日志中标识该类的日志信息
    private static final String TAG = "Contact";

    // 查询联系人的 SQL 选择语句模板
    private static final String CALLER_ID_SELECTION = "PHONE_NUMBERS_EQUAL(" + Phone.NUMBER
            + ",?) AND " + Data.MIMETYPE + "='" + Phone.CONTENT_ITEM_TYPE + "'"
            + " AND " + Data.RAW_CONTACT_ID + " IN "
            + "(SELECT raw_contact_id "
            + " FROM phone_lookup"
            + " WHERE min_match = '+')";

    /**
     * 根据电话号码从设备联系人中获取联系人姓名。
     * 如果联系人姓名已经在缓存中，则直接从缓存中获取；
     * 否则，执行数据库查询，并将结果存入缓存。
     *
     * @param context     应用程序上下文，用于访问内容解析器
     * @param phoneNumber 要查询的电话号码
     * @return 联系人姓名，如果未找到匹配的联系人则返回 null
     */
    public static String getContact(Context context, String phoneNumber) {
        // 如果联系人缓存还未初始化，则创建一个新的 HashMap 用于存储缓存信息
        if(sContactCache == null) {
            sContactCache = new HashMap<String, String>();
        }

        // 检查缓存中是否已经存在该电话号码对应的联系人姓名
        if(sContactCache.containsKey(phoneNumber)) {
            // 如果存在，则直接从缓存中返回联系人姓名
            return sContactCache.get(phoneNumber);
        }

        // 将查询模板中的占位符 "+" 替换为电话号码的最小匹配格式
        String selection = CALLER_ID_SELECTION.replace("+",
                PhoneNumberUtils.toCallerIDMinMatch(phoneNumber));

        // 使用内容解析器查询联系人数据库
        Cursor cursor = context.getContentResolver().query(
                Data.CONTENT_URI,  // 查询的数据 URI
                new String [] { Phone.DISPLAY_NAME },  // 查询返回的列，这里只需要联系人姓名
                selection,  // 查询的选择条件
                new String[] { phoneNumber },  // 选择条件的参数
                null);  // 排序规则，这里使用默认排序

        // 如果查询结果不为空且游标可以移动到第一行
        if (cursor != null && cursor.moveToFirst()) {
            try {
                // 从游标中获取联系人姓名
                String name = cursor.getString(0);
                // 将电话号码和对应的联系人姓名存入缓存
                sContactCache.put(phoneNumber, name);
                // 返回联系人姓名
                return name;
            } catch (IndexOutOfBoundsException e) {
                // 如果获取联系人姓名时发生索引越界异常，记录错误日志
                Log.e(TAG, " Cursor get string error " + e.toString());
                // 返回 null 表示未找到联系人姓名
                return null;
            } finally {
                // 无论查询结果如何，最后都要关闭游标以释放资源
                cursor.close();
            }
        } else {
            // 如果查询结果为空，记录日志表示未找到匹配的联系人
            Log.d(TAG, "No contact matched with number:" + phoneNumber);
            // 返回 null 表示未找到联系人姓名
            return null;
        }
    }
}