/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// 声明包名
package net.micode.notes.data;

// 导入必要的类
import android.app.SearchManager;
import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Intent;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.R;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.NotesDatabaseHelper.TABLE;

/**
 * NotesProvider 类，继承自 ContentProvider，用于管理笔记数据的访问和操作。
 * 它提供了对笔记和笔记数据的增删改查功能，同时支持搜索功能。
 */
public class NotesProvider extends ContentProvider {
    // 用于匹配不同的 URI，以便根据不同的 URI 执行不同的操作
    private static final UriMatcher mMatcher;
    // 用于操作 SQLite 数据库的帮助类实例
    private NotesDatabaseHelper mHelper;
    // 日志标签，用于在日志中标识该类的信息
    private static final String TAG = "NotesProvider";
    // 定义不同 URI 匹配的常量
    private static final int URI_NOTE            = 1;
    private static final int URI_NOTE_ITEM       = 2;
    private static final int URI_DATA            = 3;
    private static final int URI_DATA_ITEM       = 4;
    private static final int URI_SEARCH          = 5;
    private static final int URI_SEARCH_SUGGEST  = 6;

    // 静态代码块，在类加载时执行，初始化 UriMatcher
    static {
        mMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        // 添加不同的 URI 匹配规则
        mMatcher.addURI(Notes.AUTHORITY, "note", URI_NOTE);
        mMatcher.addURI(Notes.AUTHORITY, "note/#", URI_NOTE_ITEM);
        mMatcher.addURI(Notes.AUTHORITY, "data", URI_DATA);
        mMatcher.addURI(Notes.AUTHORITY, "data/#", URI_DATA_ITEM);
        mMatcher.addURI(Notes.AUTHORITY, "search", URI_SEARCH);
        mMatcher.addURI(Notes.AUTHORITY, SearchManager.SUGGEST_URI_PATH_QUERY, URI_SEARCH_SUGGEST);
        mMatcher.addURI(Notes.AUTHORITY, SearchManager.SUGGEST_URI_PATH_QUERY + "/*", URI_SEARCH_SUGGEST);
    }

    /**
     * 用于搜索笔记的投影，定义了搜索结果的列。
     * x'0A' 表示 SQLite 中的 '\n' 字符，为了在搜索结果中显示更多信息，会去除标题和内容中的 '\n' 和空格。
     */
    private static final String NOTES_SEARCH_PROJECTION = NoteColumns.ID + ","
            + NoteColumns.ID + " AS " + SearchManager.SUGGEST_COLUMN_INTENT_EXTRA_DATA + ","
            + "TRIM(REPLACE(" + NoteColumns.SNIPPET + ", x'0A','')) AS " + SearchManager.SUGGEST_COLUMN_TEXT_1 + ","
            + "TRIM(REPLACE(" + NoteColumns.SNIPPET + ", x'0A','')) AS " + SearchManager.SUGGEST_COLUMN_TEXT_2 + ","
            + R.drawable.search_result + " AS " + SearchManager.SUGGEST_COLUMN_ICON_1 + ","
            + "'" + Intent.ACTION_VIEW + "' AS " + SearchManager.SUGGEST_COLUMN_INTENT_ACTION + ","
            + "'" + Notes.TextNote.CONTENT_TYPE + "' AS " + SearchManager.SUGGEST_COLUMN_INTENT_DATA;

    /**
     * 用于搜索笔记摘要的 SQL 查询语句。
     * 从笔记表中查询符合条件的笔记，条件为摘要包含搜索字符串，且父 ID 不等于回收站文件夹 ID，类型为笔记。
     */
    private static String NOTES_SNIPPET_SEARCH_QUERY = "SELECT " + NOTES_SEARCH_PROJECTION
            + " FROM " + TABLE.NOTE
            + " WHERE " + NoteColumns.SNIPPET + " LIKE ?"
            + " AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER
            + " AND " + NoteColumns.TYPE + "=" + Notes.TYPE_NOTE;

    /**
     * 当 ContentProvider 创建时调用，初始化 NotesDatabaseHelper 实例。
     * @return 总是返回 true，表示 ContentProvider 创建成功。
     */
    @Override
    public boolean onCreate() {
        mHelper = NotesDatabaseHelper.getInstance(getContext());
        return true;
    }

    /**
     * 根据给定的 URI 查询数据。
     * @param uri 查询的 URI，用于指定查询的资源。
     * @param projection 查询的列，指定要返回的列。
     * @param selection 查询的条件，用于筛选数据。
     * @param selectionArgs 查询条件的参数。
     * @param sortOrder 查询结果的排序方式。
     * @return 返回包含查询结果的 Cursor 对象。
     */
    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs,
                        String sortOrder) {
        Cursor c = null;
        // 获取可读的 SQLite 数据库实例
        SQLiteDatabase db = mHelper.getReadableDatabase();
        String id = null;
        // 根据 URI 匹配结果执行不同的查询操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE:
                // 查询笔记表
                c = db.query(TABLE.NOTE, projection, selection, selectionArgs, null, null,
                        sortOrder);
                break;
            case URI_NOTE_ITEM:
                // 获取笔记的 ID
                id = uri.getPathSegments().get(1);
                // 根据笔记 ID 查询笔记表
                c = db.query(TABLE.NOTE, projection, NoteColumns.ID + "=" + id
                        + parseSelection(selection), selectionArgs, null, null, sortOrder);
                break;
            case URI_DATA:
                // 查询数据记录表
                c = db.query(TABLE.DATA, projection, selection, selectionArgs, null, null,
                        sortOrder);
                break;
            case URI_DATA_ITEM:
                // 获取数据记录的 ID
                id = uri.getPathSegments().get(1);
                // 根据数据记录 ID 查询数据记录表
                c = db.query(TABLE.DATA, projection, DataColumns.ID + "=" + id
                        + parseSelection(selection), selectionArgs, null, null, sortOrder);
                break;
            case URI_SEARCH:
            case URI_SEARCH_SUGGEST:
                // 搜索操作不允许指定排序、投影等参数
                if (sortOrder != null || projection != null) {
                    throw new IllegalArgumentException(
                            "do not specify sortOrder, selection, selectionArgs, or projection" + "with this query");
                }

                String searchString = null;
                if (mMatcher.match(uri) == URI_SEARCH_SUGGEST) {
                    // 从 URI 路径中获取搜索字符串
                    if (uri.getPathSegments().size() > 1) {
                        searchString = uri.getPathSegments().get(1);
                    }
                } else {
                    // 从 URI 查询参数中获取搜索字符串
                    searchString = uri.getQueryParameter("pattern");
                }

                // 如果搜索字符串为空，返回 null
                if (TextUtils.isEmpty(searchString)) {
                    return null;
                }

                try {
                    // 格式化搜索字符串，添加通配符
                    searchString = String.format("%%%s%%", searchString);
                    // 执行搜索查询
                    c = db.rawQuery(NOTES_SNIPPET_SEARCH_QUERY,
                            new String[] { searchString });
                } catch (IllegalStateException ex) {
                    // 记录异常信息
                    Log.e(TAG, "got exception: " + ex.toString());
                }
                break;
            default:
                // 未知的 URI 抛出异常
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
        if (c != null) {
            // 设置通知 URI，当数据变化时通知 ContentResolver
            c.setNotificationUri(getContext().getContentResolver(), uri);
        }
        return c;
    }

    /**
     * 向指定的 URI 插入数据。
     * @param uri 插入数据的目标 URI。
     * @param values 要插入的数据。
     * @return 返回插入数据后的 URI。
     */
    @Override
    public Uri insert(Uri uri, ContentValues values) {
        // 获取可写的 SQLite 数据库实例
        SQLiteDatabase db = mHelper.getWritableDatabase();
        long dataId = 0, noteId = 0, insertedId = 0;
        // 根据 URI 匹配结果执行不同的插入操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE:
                // 插入笔记数据
                insertedId = noteId = db.insert(TABLE.NOTE, null, values);
                break;
            case URI_DATA:
                if (values.containsKey(DataColumns.NOTE_ID)) {
                    // 获取笔记 ID
                    noteId = values.getAsLong(DataColumns.NOTE_ID);
                } else {
                    // 记录缺少笔记 ID 的错误信息
                    Log.d(TAG, "Wrong data format without note id:" + values.toString());
                }
                // 插入数据记录
                insertedId = dataId = db.insert(TABLE.DATA, null, values);
                break;
            default:
                // 未知的 URI 抛出异常
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
        // 通知笔记 URI 数据变化
        if (noteId > 0) {
            getContext().getContentResolver().notifyChange(
                    ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), null);
        }

        // 通知数据记录 URI 数据变化
        if (dataId > 0) {
            getContext().getContentResolver().notifyChange(
                    ContentUris.withAppendedId(Notes.CONTENT_DATA_URI, dataId), null);
        }

        return ContentUris.withAppendedId(uri, insertedId);
    }

    /**
     * 根据给定的 URI 删除数据。
     * @param uri 删除数据的目标 URI。
     * @param selection 删除的条件。
     * @param selectionArgs 删除条件的参数。
     * @return 返回删除的数据行数。
     */
    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        int count = 0;
        String id = null;
        // 获取可写的 SQLite 数据库实例
        SQLiteDatabase db = mHelper.getWritableDatabase();
        boolean deleteData = false;
        // 根据 URI 匹配结果执行不同的删除操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE:
                // 构建删除条件
                selection = "(" + selection + ") AND " + NoteColumns.ID + ">0 ";
                // 删除笔记数据
                count = db.delete(TABLE.NOTE, selection, selectionArgs);
                break;
            case URI_NOTE_ITEM:
                // 获取笔记的 ID
                id = uri.getPathSegments().get(1);
                /**
                 * ID 小于 0 的是系统文件夹，不允许删除到回收站
                 */
                long noteId = Long.valueOf(id);
                if (noteId <= 0) {
                    break;
                }
                // 根据笔记 ID 删除笔记数据
                count = db.delete(TABLE.NOTE,
                        NoteColumns.ID + "=" + id + parseSelection(selection), selectionArgs);
                break;
            case URI_DATA:
                // 删除数据记录
                count = db.delete(TABLE.DATA, selection, selectionArgs);
                deleteData = true;
                break;
            case URI_DATA_ITEM:
                // 获取数据记录的 ID
                id = uri.getPathSegments().get(1);
                // 根据数据记录 ID 删除数据记录
                count = db.delete(TABLE.DATA,
                        DataColumns.ID + "=" + id + parseSelection(selection), selectionArgs);
                deleteData = true;
                break;
            default:
                // 未知的 URI 抛出异常
                throw new IllegalArgumentException("Unknown URI " + uri);
        }
        if (count > 0) {
            if (deleteData) {
                // 通知笔记 URI 数据变化
                getContext().getContentResolver().notifyChange(Notes.CONTENT_NOTE_URI, null);
            }
            // 通知当前 URI 数据变化
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return count;
    }

    /**
     * 根据给定的 URI 更新数据。
     * @param uri 更新数据的目标 URI。
     * @param values 要更新的数据。
     * @param selection 更新的条件。
     * @param selectionArgs 更新条件的参数。
     * @return 返回更新的数据行数。
     */
    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        int count = 0;
        String id = null;
        // 获取可写的 SQLite 数据库实例
        SQLiteDatabase db = mHelper.getWritableDatabase();
        boolean updateData = false;
        // 根据 URI 匹配结果执行不同的更新操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE:
                // 增加笔记版本号
                increaseNoteVersion(-1, selection, selectionArgs);
                // 更新笔记数据
                count = db.update(TABLE.NOTE, values, selection, selectionArgs);
                break;
            case URI_NOTE_ITEM:
                // 获取笔记的 ID
                id = uri.getPathSegments().get(1);
                // 增加指定笔记的版本号
                increaseNoteVersion(Long.valueOf(id), selection, selectionArgs);
                // 根据笔记 ID 更新笔记数据
                count = db.update(TABLE.NOTE, values, NoteColumns.ID + "=" + id
                        + parseSelection(selection), selectionArgs);
                break;
            case URI_DATA:
                // 更新数据记录
                count = db.update(TABLE.DATA, values, selection, selectionArgs);
                updateData = true;
                break;
            case URI_DATA_ITEM:
                // 获取数据记录的 ID
                id = uri.getPathSegments().get(1);
                // 根据数据记录 ID 更新数据记录
                count = db.update(TABLE.DATA, values, DataColumns.ID + "=" + id
                        + parseSelection(selection), selectionArgs);
                updateData = true;
                break;
            default:
                // 未知的 URI 抛出异常
                throw new IllegalArgumentException("Unknown URI " + uri);
        }

        if (count > 0) {
            if (updateData) {
                // 通知笔记 URI 数据变化
                getContext().getContentResolver().notifyChange(Notes.CONTENT_NOTE_URI, null);
            }
            // 通知当前 URI 数据变化
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return count;
    }

    /**
     * 解析查询条件，添加 AND 关键字和括号。
     * @param selection 原始的查询条件。
     * @return 解析后的查询条件。
     */
    private String parseSelection(String selection) {
        return (!TextUtils.isEmpty(selection) ? " AND (" + selection + ')' : "");
    }

    /**
     * 增加笔记的版本号。
     * @param id 笔记的 ID，如果为 -1 则更新所有符合条件的笔记。
     * @param selection 更新的条件。
     * @param selectionArgs 更新条件的参数。
     */
    private void increaseNoteVersion(long id, String selection, String[] selectionArgs) {
        StringBuilder sql = new StringBuilder(120);
        sql.append("UPDATE ");
        sql.append(TABLE.NOTE);
        sql.append(" SET ");
        sql.append(NoteColumns.VERSION);
        sql.append("=" + NoteColumns.VERSION + "+1 ");

        if (id > 0 || !TextUtils.isEmpty(selection)) {
            sql.append(" WHERE ");
        }
        if (id > 0) {
            sql.append(NoteColumns.ID + "=" + String.valueOf(id));
        }
        if (!TextUtils.isEmpty(selection)) {
            String selectString = id > 0 ? parseSelection(selection) : selection;
            for (String args : selectionArgs) {
                selectString = selectString.replaceFirst("\\?", args);
            }
            sql.append(selectString);
        }

        // 执行 SQL 语句更新笔记版本号
        mHelper.getWritableDatabase().execSQL(sql.toString());
    }

    /**
     * 获取给定 URI 的 MIME 类型。
     * 目前该方法未实现，返回 null。
     * @param uri 要获取 MIME 类型的 URI。
     * @return 目前返回 null。
     */
    @Override
    public String getType(Uri uri) {
        // TODO Auto-generated method stub
        return null;
    }

}