/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * MetaData 类继承自 Task 类，用于处理与 Google 任务相关的元数据。
 * 该类包含设置元数据、获取相关 Google 任务 ID 等功能。
 */
public class MetaData extends Task {
    // 日志标签，用于标识该类的日志信息
    private final static String TAG = MetaData.class.getSimpleName();

    // 存储相关的 Google 任务 ID
    private String mRelatedGid = null;

    /**
     * 设置元数据信息。
     * 将给定的 Google 任务 ID 和元数据信息组合并存储到笔记中。
     *
     * @param gid 相关的 Google 任务 ID
     * @param metaInfo 包含元数据信息的 JSONObject
     */
    public void setMeta(String gid, JSONObject metaInfo) {
        try {
            // 将相关的 Google 任务 ID 放入元数据信息中
            metaInfo.put(GTaskStringUtils.META_HEAD_GTASK_ID, gid);
        } catch (JSONException e) {
            // 记录错误日志，表明无法将相关的 Google 任务 ID 放入元数据信息中
            Log.e(TAG, "failed to put related gid");
        }
        // 将元数据信息转换为字符串并设置为笔记内容
        setNotes(metaInfo.toString());
        // 设置笔记的名称为元数据笔记的名称
        setName(GTaskStringUtils.META_NOTE_NAME);
    }

    /**
     * 获取相关的 Google 任务 ID。
     *
     * @return 相关的 Google 任务 ID，如果未设置则返回 null
     */
    public String getRelatedGid() {
        return mRelatedGid;
    }

    /**
     * 判断该元数据是否值得保存。
     * 如果笔记内容不为空，则认为值得保存。
     *
     * @return 如果笔记内容不为空则返回 true，否则返回 false
     */
    @Override
    public boolean isWorthSaving() {
        return getNotes() != null;
    }

    /**
     * 根据远程 JSON 对象设置元数据内容。
     * 调用父类的方法设置内容，并从笔记中提取相关的 Google 任务 ID。
     *
     * @param js 包含远程元数据信息的 JSONObject
     */
    @Override
    public void setContentByRemoteJSON(JSONObject js) {
        // 调用父类的方法设置内容
        super.setContentByRemoteJSON(js);
        if (getNotes() != null) {
            try {
                // 将笔记内容转换为 JSONObject
                JSONObject metaInfo = new JSONObject(getNotes().trim());
                // 从元数据信息中提取相关的 Google 任务 ID
                mRelatedGid = metaInfo.getString(GTaskStringUtils.META_HEAD_GTASK_ID);
            } catch (JSONException e) {
                // 记录警告日志，表明无法从笔记中提取相关的 Google 任务 ID
                Log.w(TAG, "failed to get related gid");
                mRelatedGid = null;
            }
        }
    }

    /**
     * 根据本地 JSON 对象设置元数据内容。
     * 该方法不应该被调用，因为元数据不应该通过本地 JSON 设置。
     *
     * @param js 包含本地元数据信息的 JSONObject
     * @throws IllegalAccessError 始终抛出该异常，表明该方法不应该被调用
     */
    @Override
    public void setContentByLocalJSON(JSONObject js) {
        // 抛出异常，表明该方法不应该被调用
        throw new IllegalAccessError("MetaData:setContentByLocalJSON should not be called");
    }

    /**
     * 从内容中获取本地 JSON 对象。
     * 该方法不应该被调用，因为元数据不应该生成本地 JSON。
     *
     * @return 无返回值，因为会抛出异常
     * @throws IllegalAccessError 始终抛出该异常，表明该方法不应该被调用
     */
    @Override
    public JSONObject getLocalJSONFromContent() {
        // 抛出异常，表明该方法不应该被调用
        throw new IllegalAccessError("MetaData:getLocalJSONFromContent should not be called");
    }

    /**
     * 获取同步操作。
     * 该方法不应该被调用，因为元数据不参与同步操作。
     *
     * @param c 用于查询同步操作的 Cursor
     * @return 无返回值，因为会抛出异常
     * @throws IllegalAccessError 始终抛出该异常，表明该方法不应该被调用
     */
    @Override
    public int getSyncAction(Cursor c) {
        // 抛出异常，表明该方法不应该被调用
        throw new IllegalAccessError("MetaData:getSyncAction should not be called");
    }
}