/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;

import org.json.JSONObject;

/**
 * Node 类是一个抽象类，代表同步操作中的节点。
 * 它定义了同步操作的常量和节点的基本属性，如全局ID、名称、最后修改时间和删除状态。
 * 同时，它提供了一些抽象方法，需要子类去实现具体的同步操作逻辑。
 */
public abstract class Node {
    // 表示无同步操作
    public static final int SYNC_ACTION_NONE = 0;
    // 表示在远程添加节点的同步操作
    public static final int SYNC_ACTION_ADD_REMOTE = 1;
    // 表示在本地添加节点的同步操作
    public static final int SYNC_ACTION_ADD_LOCAL = 2;
    // 表示在远程删除节点的同步操作
    public static final int SYNC_ACTION_DEL_REMOTE = 3;
    // 表示在本地删除节点的同步操作
    public static final int SYNC_ACTION_DEL_LOCAL = 4;
    // 表示更新远程节点的同步操作
    public static final int SYNC_ACTION_UPDATE_REMOTE = 5;
    // 表示更新本地节点的同步操作
    public static final int SYNC_ACTION_UPDATE_LOCAL = 6;
    // 表示同步更新时发生冲突的情况
    public static final int SYNC_ACTION_UPDATE_CONFLICT = 7;
    // 表示同步操作出现错误的情况
    public static final int SYNC_ACTION_ERROR = 8;

    // 节点的全局唯一标识符
    private String mGid;
    // 节点的名称
    private String mName;
    // 节点的最后修改时间
    private long mLastModified;
    // 节点是否已被删除的标志
    private boolean mDeleted;

    /**
     * 构造函数，初始化节点的属性。
     */
    public Node() {
        // 初始全局ID为 null
        mGid = null;
        // 初始名称为空字符串
        mName = "";
        // 初始最后修改时间为 0
        mLastModified = 0;
        // 初始删除状态为未删除
        mDeleted = false;
    }

    /**
     * 获取创建节点的同步操作的 JSON 对象。
     * 具体实现由子类完成。
     *
     * @param actionId 同步操作的 ID
     * @return 包含创建操作信息的 JSON 对象
     */
    public abstract JSONObject getCreateAction(int actionId);

    /**
     * 获取更新节点的同步操作的 JSON 对象。
     * 具体实现由子类完成。
     *
     * @param actionId 同步操作的 ID
     * @return 包含更新操作信息的 JSON 对象
     */
    public abstract JSONObject getUpdateAction(int actionId);

    /**
     * 根据远程的 JSON 对象设置节点的内容。
     * 具体实现由子类完成。
     *
     * @param js 包含远程节点信息的 JSON 对象
     */
    public abstract void setContentByRemoteJSON(JSONObject js);

    /**
     * 根据本地的 JSON 对象设置节点的内容。
     * 具体实现由子类完成。
     *
     * @param js 包含本地节点信息的 JSON 对象
     */
    public abstract void setContentByLocalJSON(JSONObject js);

    /**
     * 从节点内容生成本地的 JSON 对象。
     * 具体实现由子类完成。
     *
     * @return 包含本地节点信息的 JSON 对象
     */
    public abstract JSONObject getLocalJSONFromContent();

    /**
     * 根据数据库游标获取同步操作类型。
     * 具体实现由子类完成。
     *
     * @param c 数据库游标
     * @return 同步操作类型，对应上述定义的常量
     */
    public abstract int getSyncAction(Cursor c);

    /**
     * 设置节点的全局唯一标识符。
     *
     * @param gid 要设置的全局唯一标识符
     */
    public void setGid(String gid) {
        this.mGid = gid;
    }

    /**
     * 设置节点的名称。
     *
     * @param name 要设置的节点名称
     */
    public void setName(String name) {
        this.mName = name;
    }

    /**
     * 设置节点的最后修改时间。
     *
     * @param lastModified 要设置的最后修改时间
     */
    public void setLastModified(long lastModified) {
        this.mLastModified = lastModified;
    }

    /**
     * 设置节点的删除状态。
     *
     * @param deleted 要设置的删除状态，true 表示已删除，false 表示未删除
     */
    public void setDeleted(boolean deleted) {
        this.mDeleted = deleted;
    }

    /**
     * 获取节点的全局唯一标识符。
     *
     * @return 节点的全局唯一标识符
     */
    public String getGid() {
        return this.mGid;
    }

    /**
     * 获取节点的名称。
     *
     * @return 节点的名称
     */
    public String getName() {
        return this.mName;
    }

    /**
     * 获取节点的最后修改时间。
     *
     * @return 节点的最后修改时间
     */
    public long getLastModified() {
        return this.mLastModified;
    }

    /**
     * 获取节点的删除状态。
     *
     * @return 节点的删除状态，true 表示已删除，false 表示未删除
     */
    public boolean getDeleted() {
        return this.mDeleted;
    }
}