/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.NotesDatabaseHelper.TABLE;
import net.micode.notes.gtask.exception.ActionFailureException;

import org.json.JSONException;
import org.json.JSONObject;

/**
 * SqlData 类用于处理与笔记数据相关的数据库操作，
 * 支持从数据库加载数据、根据 JSON 设置数据、将数据保存到数据库等功能。
 */
public class SqlData {
    // 日志标签，用于标识该类的日志信息
    private static final String TAG = SqlData.class.getSimpleName();
    // 无效的 ID，用于初始化和判断数据状态
    private static final int INVALID_ID = -99999;
    // 用于查询数据的列投影数组
    public static final String[] PROJECTION_DATA = new String[] {
            DataColumns.ID, DataColumns.MIME_TYPE, DataColumns.CONTENT, DataColumns.DATA1,
            DataColumns.DATA3
    };
    // 数据 ID 列在投影数组中的索引
    public static final int DATA_ID_COLUMN = 0;
    // 数据 MIME 类型列在投影数组中的索引
    public static final int DATA_MIME_TYPE_COLUMN = 1;
    // 数据内容列在投影数组中的索引
    public static final int DATA_CONTENT_COLUMN = 2;
    // 数据内容的 DATA1 列在投影数组中的索引
    public static final int DATA_CONTENT_DATA_1_COLUMN = 3;
    // 数据内容的 DATA3 列在投影数组中的索引
    public static final int DATA_CONTENT_DATA_3_COLUMN = 4;
    // 内容解析器，用于与内容提供者交互
    private ContentResolver mContentResolver;
    // 标记数据是否为新创建
    private boolean mIsCreate;
    // 数据的 ID
    private long mDataId;
    // 数据的 MIME 类型
    private String mDataMimeType;
    // 数据的内容
    private String mDataContent;
    // 数据内容的 DATA1 字段
    private long mDataContentData1;
    // 数据内容的 DATA3 字段
    private String mDataContentData3;
    // 存储数据差异的 ContentValues 对象
    private ContentValues mDiffDataValues;

    /**
     * 构造函数，用于创建一个新的 SqlData 对象。
     *
     * @param context 上下文对象，用于获取 ContentResolver
     */
    public SqlData(Context context) {
        // 获取 ContentResolver 用于与内容提供者交互
        mContentResolver = context.getContentResolver();
        // 标记为新创建的数据
        mIsCreate = true;
        // 初始化数据 ID 为无效值
        mDataId = INVALID_ID;
        // 初始化数据 MIME 类型为默认的笔记类型
        mDataMimeType = DataConstants.NOTE;
        // 初始化数据内容为空字符串
        mDataContent = "";
        // 初始化数据内容的 DATA1 字段为 0
        mDataContentData1 = 0;
        // 初始化数据内容的 DATA3 字段为空字符串
        mDataContentData3 = "";
        // 初始化存储数据差异的 ContentValues 对象
        mDiffDataValues = new ContentValues();
    }

    /**
     * 构造函数，用于从 Cursor 加载数据创建 SqlData 对象。
     *
     * @param context 上下文对象，用于获取 ContentResolver
     * @param c       包含数据的 Cursor 对象
     */
    public SqlData(Context context, Cursor c) {
        // 获取 ContentResolver 用于与内容提供者交互
        mContentResolver = context.getContentResolver();
        // 标记为非新创建的数据
        mIsCreate = false;
        // 从 Cursor 中加载数据
        loadFromCursor(c);
        // 初始化存储数据差异的 ContentValues 对象
        mDiffDataValues = new ContentValues();
    }

    /**
     * 从 Cursor 中加载数据到当前对象。
     *
     * @param c 包含数据的 Cursor 对象
     */
    private void loadFromCursor(Cursor c) {
        // 从 Cursor 中获取数据 ID
        mDataId = c.getLong(DATA_ID_COLUMN);
        // 从 Cursor 中获取数据 MIME 类型
        mDataMimeType = c.getString(DATA_MIME_TYPE_COLUMN);
        // 从 Cursor 中获取数据内容
        mDataContent = c.getString(DATA_CONTENT_COLUMN);
        // 从 Cursor 中获取数据内容的 DATA1 字段
        mDataContentData1 = c.getLong(DATA_CONTENT_DATA_1_COLUMN);
        // 从 Cursor 中获取数据内容的 DATA3 字段
        mDataContentData3 = c.getString(DATA_CONTENT_DATA_3_COLUMN);
    }

    /**
     * 根据 JSON 对象设置数据内容，并记录差异。
     *
     * @param js 包含数据的 JSON 对象
     * @throws JSONException 如果 JSON 解析出错
     */
    public void setContent(JSONObject js) throws JSONException {
        // 从 JSON 对象中获取数据 ID，如果不存在则使用无效值
        long dataId = js.has(DataColumns.ID) ? js.getLong(DataColumns.ID) : INVALID_ID;
        // 如果是新创建的数据或者数据 ID 发生变化，记录差异
        if (mIsCreate || mDataId != dataId) {
            mDiffDataValues.put(DataColumns.ID, dataId);
        }
        // 更新数据 ID
        mDataId = dataId;

        // 从 JSON 对象中获取数据 MIME 类型，如果不存在则使用默认值
        String dataMimeType = js.has(DataColumns.MIME_TYPE) ? js.getString(DataColumns.MIME_TYPE)
                : DataConstants.NOTE;
        // 如果是新创建的数据或者 MIME 类型发生变化，记录差异
        if (mIsCreate || !mDataMimeType.equals(dataMimeType)) {
            mDiffDataValues.put(DataColumns.MIME_TYPE, dataMimeType);
        }
        // 更新数据 MIME 类型
        mDataMimeType = dataMimeType;

        // 从 JSON 对象中获取数据内容，如果不存在则使用空字符串
        String dataContent = js.has(DataColumns.CONTENT) ? js.getString(DataColumns.CONTENT) : "";
        // 如果是新创建的数据或者数据内容发生变化，记录差异
        if (mIsCreate || !mDataContent.equals(dataContent)) {
            mDiffDataValues.put(DataColumns.CONTENT, dataContent);
        }
        // 更新数据内容
        mDataContent = dataContent;

        // 从 JSON 对象中获取数据内容的 DATA1 字段，如果不存在则使用 0
        long dataContentData1 = js.has(DataColumns.DATA1) ? js.getLong(DataColumns.DATA1) : 0;
        // 如果是新创建的数据或者 DATA1 字段发生变化，记录差异
        if (mIsCreate || mDataContentData1 != dataContentData1) {
            mDiffDataValues.put(DataColumns.DATA1, dataContentData1);
        }
        // 更新数据内容的 DATA1 字段
        mDataContentData1 = dataContentData1;

        // 从 JSON 对象中获取数据内容的 DATA3 字段，如果不存在则使用空字符串
        String dataContentData3 = js.has(DataColumns.DATA3) ? js.getString(DataColumns.DATA3) : "";
        // 如果是新创建的数据或者 DATA3 字段发生变化，记录差异
        if (mIsCreate || !mDataContentData3.equals(dataContentData3)) {
            mDiffDataValues.put(DataColumns.DATA3, dataContentData3);
        }
        // 更新数据内容的 DATA3 字段
        mDataContentData3 = dataContentData3;
    }

    /**
     * 获取当前数据的 JSON 表示。
     *
     * @return 包含当前数据的 JSON 对象
     * @throws JSONException 如果 JSON 构建出错
     */
    public JSONObject getContent() throws JSONException {
        // 如果是新创建的数据且未保存到数据库，记录错误日志并返回 null
        if (mIsCreate) {
            Log.e(TAG, "it seems that we haven't created this in database yet");
            return null;
        }
        // 创建一个新的 JSON 对象
        JSONObject js = new JSONObject();
        // 将数据 ID 放入 JSON 对象
        js.put(DataColumns.ID, mDataId);
        // 将数据 MIME 类型放入 JSON 对象
        js.put(DataColumns.MIME_TYPE, mDataMimeType);
        // 将数据内容放入 JSON 对象
        js.put(DataColumns.CONTENT, mDataContent);
        // 将数据内容的 DATA1 字段放入 JSON 对象
        js.put(DataColumns.DATA1, mDataContentData1);
        // 将数据内容的 DATA3 字段放入 JSON 对象
        js.put(DataColumns.DATA3, mDataContentData3);
        return js;
    }

    /**
     * 将数据提交到数据库，根据数据状态进行插入或更新操作。
     *
     * @param noteId         关联的笔记 ID
     * @param validateVersion 是否验证版本
     * @param version        版本号
     */
    public void commit(long noteId, boolean validateVersion, long version) {
        // 如果是新创建的数据
        if (mIsCreate) {
            // 如果数据 ID 无效且差异值中包含数据 ID，移除该差异
            if (mDataId == INVALID_ID && mDiffDataValues.containsKey(DataColumns.ID)) {
                mDiffDataValues.remove(DataColumns.ID);
            }
            // 将关联的笔记 ID 放入差异值
            mDiffDataValues.put(DataColumns.NOTE_ID, noteId);
            // 插入数据到数据库，并获取插入后的 Uri
            Uri uri = mContentResolver.insert(Notes.CONTENT_DATA_URI, mDiffDataValues);
            try {
                // 从 Uri 中获取插入后的新数据 ID
                mDataId = Long.valueOf(uri.getPathSegments().get(1));
            } catch (NumberFormatException e) {
                // 记录错误日志并抛出异常
                Log.e(TAG, "Get note id error :" + e.toString());
                throw new ActionFailureException("create note failed");
            }
        } else {
            // 如果不是新创建的数据且有差异需要更新
            if (mDiffDataValues.size() > 0) {
                int result = 0;
                if (!validateVersion) {
                    // 不验证版本时，直接更新数据
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues, null, null);
                } else {
                    // 验证版本时，根据版本条件更新数据
                    result = mContentResolver.update(ContentUris.withAppendedId(
                                    Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues,
                            " ? in (SELECT " + NoteColumns.ID + " FROM " + TABLE.NOTE
                                    + " WHERE " + NoteColumns.VERSION + "=?)", new String[] {
                                    String.valueOf(noteId), String.valueOf(version)
                            });
                }
                // 如果更新结果为 0，记录警告日志
                if (result == 0) {
                    Log.w(TAG, "there is no update. maybe user updates note when syncing");
                }
            }
        }
        // 清空差异值
        mDiffDataValues.clear();
        // 标记数据为非新创建
        mIsCreate = false;
    }

    /**
     * 获取数据的 ID。
     *
     * @return 数据的 ID
     */
    public long getId() {
        return mDataId;
    }
}