/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.gtask.exception.ActionFailureException;
import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Task类继承自Node类，代表Google任务中的一个任务。
 * 它负责处理任务的创建、更新操作，从JSON数据中设置任务内容，
 * 生成本地JSON数据，以及判断同步操作类型等功能。
 */
public class Task extends Node {
    // 日志标签，用于标识该类的日志信息
    private static final String TAG = Task.class.getSimpleName();
    // 任务是否完成的标志
    private boolean mCompleted;
    // 任务的备注信息
    private String mNotes;
    // 任务的元信息，以JSON对象形式存储
    private JSONObject mMetaInfo;
    // 该任务的前一个兄弟任务
    private Task mPriorSibling;
    // 该任务所属的任务列表
    private TaskList mParent;

    /**
     * 构造函数，初始化任务的各项属性。
     */
    public Task() {
        // 调用父类的构造函数
        super();
        // 初始化任务为未完成状态
        mCompleted = false;
        // 初始化备注信息为空
        mNotes = null;
        // 初始化前一个兄弟任务为空
        mPriorSibling = null;
        // 初始化所属任务列表为空
        mParent = null;
        // 初始化元信息为空
        mMetaInfo = null;
    }

    /**
     * 生成创建任务的JSON操作对象。
     *
     * @param actionId 操作的唯一标识符
     * @return 包含创建任务操作信息的JSON对象
     * @throws ActionFailureException 如果生成JSON对象时发生异常
     */
    public JSONObject getCreateAction(int actionId) {
        // 创建一个新的JSON对象
        JSONObject js = new JSONObject();

        try {
            // 设置操作类型为创建任务
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_CREATE);
            // 设置操作的唯一标识符
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);
            // 设置任务在父任务列表中的索引
            js.put(GTaskStringUtils.GTASK_JSON_INDEX, mParent.getChildTaskIndex(this));

            // 创建实体差异的JSON对象
            JSONObject entity = new JSONObject();
            // 设置任务的名称
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // 设置任务的创建者ID为null
            entity.put(GTaskStringUtils.GTASK_JSON_CREATOR_ID, "null");
            // 设置实体类型为任务
            entity.put(GTaskStringUtils.GTASK_JSON_ENTITY_TYPE,
                    GTaskStringUtils.GTASK_JSON_TYPE_TASK);
            // 如果任务有备注信息，将其添加到实体差异中
            if (getNotes() != null) {
                entity.put(GTaskStringUtils.GTASK_JSON_NOTES, getNotes());
            }
            // 将实体差异添加到操作JSON对象中
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

            // 设置父任务列表的全局ID
            js.put(GTaskStringUtils.GTASK_JSON_PARENT_ID, mParent.getGid());
            // 设置目标父任务类型为任务组
            js.put(GTaskStringUtils.GTASK_JSON_DEST_PARENT_TYPE,
                    GTaskStringUtils.GTASK_JSON_TYPE_GROUP);
            // 设置任务列表的全局ID
            js.put(GTaskStringUtils.GTASK_JSON_LIST_ID, mParent.getGid());

            // 如果存在前一个兄弟任务，设置其全局ID
            if (mPriorSibling != null) {
                js.put(GTaskStringUtils.GTASK_JSON_PRIOR_SIBLING_ID, mPriorSibling.getGid());
            }

        } catch (JSONException e) {
            // 记录错误日志并抛出异常
            Log.e(TAG, e.toString());
            e.printStackTrace();
            throw new ActionFailureException("fail to generate task-create jsonobject");
        }

        return js;
    }

    /**
     * 生成更新任务的JSON操作对象。
     *
     * @param actionId 操作的唯一标识符
     * @return 包含更新任务操作信息的JSON对象
     * @throws ActionFailureException 如果生成JSON对象时发生异常
     */
    public JSONObject getUpdateAction(int actionId) {
        // 创建一个新的JSON对象
        JSONObject js = new JSONObject();

        try {
            // 设置操作类型为更新任务
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_UPDATE);
            // 设置操作的唯一标识符
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);
            // 设置任务的全局ID
            js.put(GTaskStringUtils.GTASK_JSON_ID, getGid());

            // 创建实体差异的JSON对象
            JSONObject entity = new JSONObject();
            // 设置任务的名称
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // 如果任务有备注信息，将其添加到实体差异中
            if (getNotes() != null) {
                entity.put(GTaskStringUtils.GTASK_JSON_NOTES, getNotes());
            }
            // 设置任务是否被删除的标志
            entity.put(GTaskStringUtils.GTASK_JSON_DELETED, getDeleted());
            // 将实体差异添加到操作JSON对象中
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

        } catch (JSONException e) {
            // 记录错误日志并抛出异常
            Log.e(TAG, e.toString());
            e.printStackTrace();
            throw new ActionFailureException("fail to generate task-update jsonobject");
        }

        return js;
    }

    /**
     * 根据远程JSON数据设置任务的内容。
     *
     * @param js 包含远程任务信息的JSON对象
     * @throws ActionFailureException 如果解析JSON数据时发生异常
     */
    public void setContentByRemoteJSON(JSONObject js) {
        if (js != null) {
            try {
                // 如果JSON对象包含任务的全局ID，设置任务的全局ID
                if (js.has(GTaskStringUtils.GTASK_JSON_ID)) {
                    setGid(js.getString(GTaskStringUtils.GTASK_JSON_ID));
                }
                // 如果JSON对象包含任务的最后修改时间，设置任务的最后修改时间
                if (js.has(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED)) {
                    setLastModified(js.getLong(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED));
                }
                // 如果JSON对象包含任务的名称，设置任务的名称
                if (js.has(GTaskStringUtils.GTASK_JSON_NAME)) {
                    setName(js.getString(GTaskStringUtils.GTASK_JSON_NAME));
                }
                // 如果JSON对象包含任务的备注信息，设置任务的备注信息
                if (js.has(GTaskStringUtils.GTASK_JSON_NOTES)) {
                    setNotes(js.getString(GTaskStringUtils.GTASK_JSON_NOTES));
                }
                // 如果JSON对象包含任务是否被删除的标志，设置任务的删除状态
                if (js.has(GTaskStringUtils.GTASK_JSON_DELETED)) {
                    setDeleted(js.getBoolean(GTaskStringUtils.GTASK_JSON_DELETED));
                }
                // 如果JSON对象包含任务是否完成的标志，设置任务的完成状态
                if (js.has(GTaskStringUtils.GTASK_JSON_COMPLETED)) {
                    setCompleted(js.getBoolean(GTaskStringUtils.GTASK_JSON_COMPLETED));
                }
            } catch (JSONException e) {
                // 记录错误日志并抛出异常
                Log.e(TAG, e.toString());
                e.printStackTrace();
                throw new ActionFailureException("fail to get task content from jsonobject");
            }
        }
    }

    /**
     * 根据本地JSON数据设置任务的内容。
     *
     * @param js 包含本地任务信息的JSON对象
     */
    public void setContentByLocalJSON(JSONObject js) {
        if (js == null || !js.has(GTaskStringUtils.META_HEAD_NOTE)
                || !js.has(GTaskStringUtils.META_HEAD_DATA)) {
            // 记录警告日志，提示没有可用的信息
            Log.w(TAG, "setContentByLocalJSON: nothing is avaiable");
        }

        try {
            // 从JSON对象中获取笔记信息
            JSONObject note = js.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);
            // 从JSON对象中获取数据数组
            JSONArray dataArray = js.getJSONArray(GTaskStringUtils.META_HEAD_DATA);

            // 检查笔记类型是否为普通笔记
            if (note.getInt(NoteColumns.TYPE) != Notes.TYPE_NOTE) {
                // 记录错误日志，提示类型无效
                Log.e(TAG, "invalid type");
                return;
            }

            // 遍历数据数组
            for (int i = 0; i < dataArray.length(); i++) {
                // 获取当前数据对象
                JSONObject data = dataArray.getJSONObject(i);
                // 检查数据的MIME类型是否为笔记类型
                if (TextUtils.equals(data.getString(DataColumns.MIME_TYPE), DataConstants.NOTE)) {
                    // 设置任务的名称为数据的内容
                    setName(data.getString(DataColumns.CONTENT));
                    break;
                }
            }

        } catch (JSONException e) {
            // 记录错误日志并打印异常堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }
    }

    /**
     * 根据任务的内容生成本地JSON对象。
     *
     * @return 包含任务本地信息的JSON对象，若发生异常则返回null
     */
    public JSONObject getLocalJSONFromContent() {
        // 获取任务的名称
        String name = getName();
        try {
            if (mMetaInfo == null) {
                // 新创建的任务，从网页创建
                if (name == null) {
                    // 记录警告日志，提示笔记为空
                    Log.w(TAG, "the note seems to be an empty one");
                    return null;
                }

                // 创建一个新的JSON对象
                JSONObject js = new JSONObject();
                // 创建笔记信息的JSON对象
                JSONObject note = new JSONObject();
                // 创建数据数组的JSON对象
                JSONArray dataArray = new JSONArray();
                // 创建数据的JSON对象
                JSONObject data = new JSONObject();
                // 设置数据的内容为任务的名称
                data.put(DataColumns.CONTENT, name);
                // 将数据添加到数据数组中
                dataArray.put(data);
                // 将数据数组添加到JSON对象中
                js.put(GTaskStringUtils.META_HEAD_DATA, dataArray);
                // 设置笔记类型为普通笔记
                note.put(NoteColumns.TYPE, Notes.TYPE_NOTE);
                // 将笔记信息添加到JSON对象中
                js.put(GTaskStringUtils.META_HEAD_NOTE, note);
                return js;
            } else {
                // 已同步的任务
                // 从元信息中获取笔记信息
                JSONObject note = mMetaInfo.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);
                // 从元信息中获取数据数组
                JSONArray dataArray = mMetaInfo.getJSONArray(GTaskStringUtils.META_HEAD_DATA);

                // 遍历数据数组
                for (int i = 0; i < dataArray.length(); i++) {
                    // 获取当前数据对象
                    JSONObject data = dataArray.getJSONObject(i);
                    // 检查数据的MIME类型是否为笔记类型
                    if (TextUtils.equals(data.getString(DataColumns.MIME_TYPE), DataConstants.NOTE)) {
                        // 设置数据的内容为任务的名称
                        data.put(DataColumns.CONTENT, getName());
                        break;
                    }
                }

                // 设置笔记类型为普通笔记
                note.put(NoteColumns.TYPE, Notes.TYPE_NOTE);
                return mMetaInfo;
            }
        } catch (JSONException e) {
            // 记录错误日志并打印异常堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 设置任务的元信息。
     *
     * @param metaData 包含元信息的MetaData对象
     */
    public void setMetaInfo(MetaData metaData) {
        if (metaData != null && metaData.getNotes() != null) {
            try {
                // 将元信息的备注信息转换为JSON对象并设置为任务的元信息
                mMetaInfo = new JSONObject(metaData.getNotes());
            } catch (JSONException e) {
                // 记录警告日志并将元信息设置为空
                Log.w(TAG, e.toString());
                mMetaInfo = null;
            }
        }
    }

    /**
     * 根据数据库游标判断任务的同步操作类型。
     *
     * @param c 数据库游标
     * @return 同步操作类型的常量值
     */
    public int getSyncAction(Cursor c) {
        try {
            // 从元信息中获取笔记信息
            JSONObject noteInfo = null;
            if (mMetaInfo != null && mMetaInfo.has(GTaskStringUtils.META_HEAD_NOTE)) {
                noteInfo = mMetaInfo.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);
            }

            if (noteInfo == null) {
                // 记录警告日志，提示笔记元信息已被删除
                Log.w(TAG, "it seems that note meta has been deleted");
                return SYNC_ACTION_UPDATE_REMOTE;
            }

            if (!noteInfo.has(NoteColumns.ID)) {
                // 记录警告日志，提示远程笔记ID已被删除
                Log.w(TAG, "remote note id seems to be deleted");
                return SYNC_ACTION_UPDATE_LOCAL;
            }

            // 验证笔记ID是否匹配
            if (c.getLong(SqlNote.ID_COLUMN) != noteInfo.getLong(NoteColumns.ID)) {
                // 记录警告日志，提示笔记ID不匹配
                Log.w(TAG, "note id doesn't match");
                return SYNC_ACTION_UPDATE_LOCAL;
            }

            if (c.getInt(SqlNote.LOCAL_MODIFIED_COLUMN) == 0) {
                // 本地没有更新
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // 双方都没有更新
                    return SYNC_ACTION_NONE;
                } else {
                    // 应用远程更新到本地
                    return SYNC_ACTION_UPDATE_LOCAL;
                }
            } else {
                // 验证Google任务ID是否匹配
                if (!c.getString(SqlNote.GTASK_ID_COLUMN).equals(getGid())) {
                    // 记录错误日志，提示Google任务ID不匹配
                    Log.e(TAG, "gtask id doesn't match");
                    return SYNC_ACTION_ERROR;
                }
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // 只有本地有修改
                    return SYNC_ACTION_UPDATE_REMOTE;
                } else {
                    // 发生冲突
                    return SYNC_ACTION_UPDATE_CONFLICT;
                }
            }
        } catch (Exception e) {
            // 记录错误日志并打印异常堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }

        return SYNC_ACTION_ERROR;
    }

    /**
     * 判断任务是否值得保存。
     *
     * @return 如果任务有元信息、名称或备注信息，则返回true，否则返回false
     */
    public boolean isWorthSaving() {
        return mMetaInfo != null || (getName() != null && getName().trim().length() > 0)
                || (getNotes() != null && getNotes().trim().length() > 0);
    }

    /**
     * 设置任务的完成状态。
     *
     * @param completed 任务是否完成的标志
     */
    public void setCompleted(boolean completed) {
        this.mCompleted = completed;
    }

    /**
     * 设置任务的备注信息。
     *
     * @param notes 任务的备注信息
     */
    public void setNotes(String notes) {
        this.mNotes = notes;
    }

    /**
     * 设置任务的前一个兄弟任务。
     *
     * @param priorSibling 前一个兄弟任务
     */
    public void setPriorSibling(Task priorSibling) {
        this.mPriorSibling = priorSibling;
    }

    /**
     * 设置任务所属的任务列表。
     *
     * @param parent 任务所属的任务列表
     */
    public void setParent(TaskList parent) {
        this.mParent = parent;
    }

    /**
     * 获取任务的完成状态。
     *
     * @return 任务是否完成的标志
     */
    public boolean getCompleted() {
        return this.mCompleted;
    }

    /**
     * 获取任务的备注信息。
     *
     * @return 任务的备注信息
     */
    public String getNotes() {
        return this.mNotes;
    }

    /**
     * 获取任务的前一个兄弟任务。
     *
     * @return 前一个兄弟任务
     */
    public Task getPriorSibling() {
        return this.mPriorSibling;
    }

    /**
     * 获取任务所属的任务列表。
     *
     * @return 任务所属的任务列表
     */
    public TaskList getParent() {
        return this.mParent;
    }
}