/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.gtask.exception.ActionFailureException;
import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

/**
 * TaskList类继承自Node类，代表一个任务列表。
 * 该类负责处理任务列表的创建、更新操作，根据JSON数据设置任务列表内容，
 * 生成本地JSON数据，判断同步操作类型，以及管理任务列表中的子任务等功能。
 */
public class TaskList extends Node {
    // 日志标签，用于在日志中标识该类的相关信息
    private static final String TAG = TaskList.class.getSimpleName();
    // 任务列表在父级中的索引
    private int mIndex;
    // 存储任务列表中的子任务
    private ArrayList<Task> mChildren;

    /**
     * 构造函数，初始化任务列表的各项属性。
     */
    public TaskList() {
        // 调用父类的构造函数
        super();
        // 初始化子任务列表
        mChildren = new ArrayList<Task>();
        // 初始化索引为1
        mIndex = 1;
    }

    /**
     * 生成创建任务列表的JSON操作对象。
     *
     * @param actionId 操作的唯一标识符
     * @return 包含创建任务列表操作信息的JSON对象
     * @throws ActionFailureException 如果生成JSON对象时发生异常
     */
    public JSONObject getCreateAction(int actionId) {
        // 创建一个新的JSON对象
        JSONObject js = new JSONObject();

        try {
            // 设置操作类型为创建任务列表
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_CREATE);
            // 设置操作的唯一标识符
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);
            // 设置任务列表在父级中的索引
            js.put(GTaskStringUtils.GTASK_JSON_INDEX, mIndex);

            // 创建实体差异的JSON对象
            JSONObject entity = new JSONObject();
            // 设置任务列表的名称
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // 设置任务列表的创建者ID为null
            entity.put(GTaskStringUtils.GTASK_JSON_CREATOR_ID, "null");
            // 设置实体类型为任务组
            entity.put(GTaskStringUtils.GTASK_JSON_ENTITY_TYPE,
                    GTaskStringUtils.GTASK_JSON_TYPE_GROUP);
            // 将实体差异添加到操作JSON对象中
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

        } catch (JSONException e) {
            // 记录错误日志并抛出异常
            Log.e(TAG, e.toString());
            e.printStackTrace();
            throw new ActionFailureException("fail to generate tasklist-create jsonobject");
        }

        return js;
    }

    /**
     * 生成更新任务列表的JSON操作对象。
     *
     * @param actionId 操作的唯一标识符
     * @return 包含更新任务列表操作信息的JSON对象
     * @throws ActionFailureException 如果生成JSON对象时发生异常
     */
    public JSONObject getUpdateAction(int actionId) {
        // 创建一个新的JSON对象
        JSONObject js = new JSONObject();

        try {
            // 设置操作类型为更新任务列表
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_TYPE,
                    GTaskStringUtils.GTASK_JSON_ACTION_TYPE_UPDATE);
            // 设置操作的唯一标识符
            js.put(GTaskStringUtils.GTASK_JSON_ACTION_ID, actionId);
            // 设置任务列表的全局ID
            js.put(GTaskStringUtils.GTASK_JSON_ID, getGid());

            // 创建实体差异的JSON对象
            JSONObject entity = new JSONObject();
            // 设置任务列表的名称
            entity.put(GTaskStringUtils.GTASK_JSON_NAME, getName());
            // 设置任务列表是否被删除的标志
            entity.put(GTaskStringUtils.GTASK_JSON_DELETED, getDeleted());
            // 将实体差异添加到操作JSON对象中
            js.put(GTaskStringUtils.GTASK_JSON_ENTITY_DELTA, entity);

        } catch (JSONException e) {
            // 记录错误日志并抛出异常
            Log.e(TAG, e.toString());
            e.printStackTrace();
            throw new ActionFailureException("fail to generate tasklist-update jsonobject");
        }

        return js;
    }

    /**
     * 根据远程JSON数据设置任务列表的内容。
     *
     * @param js 包含远程任务列表信息的JSON对象
     * @throws ActionFailureException 如果解析JSON数据时发生异常
     */
    public void setContentByRemoteJSON(JSONObject js) {
        if (js != null) {
            try {
                // 如果JSON对象包含任务列表的全局ID，设置任务列表的全局ID
                if (js.has(GTaskStringUtils.GTASK_JSON_ID)) {
                    setGid(js.getString(GTaskStringUtils.GTASK_JSON_ID));
                }
                // 如果JSON对象包含任务列表的最后修改时间，设置任务列表的最后修改时间
                if (js.has(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED)) {
                    setLastModified(js.getLong(GTaskStringUtils.GTASK_JSON_LAST_MODIFIED));
                }
                // 如果JSON对象包含任务列表的名称，设置任务列表的名称
                if (js.has(GTaskStringUtils.GTASK_JSON_NAME)) {
                    setName(js.getString(GTaskStringUtils.GTASK_JSON_NAME));
                }
            } catch (JSONException e) {
                // 记录错误日志并抛出异常
                Log.e(TAG, e.toString());
                e.printStackTrace();
                throw new ActionFailureException("fail to get tasklist content from jsonobject");
            }
        }
    }

    /**
     * 根据本地JSON数据设置任务列表的内容。
     *
     * @param js 包含本地任务列表信息的JSON对象
     */
    public void setContentByLocalJSON(JSONObject js) {
        if (js == null || !js.has(GTaskStringUtils.META_HEAD_NOTE)) {
            // 记录警告日志，提示没有可用的信息
            Log.w(TAG, "setContentByLocalJSON: nothing is avaiable");
        }

        try {
            // 从JSON对象中获取文件夹信息
            JSONObject folder = js.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);

            // 检查文件夹类型是否为普通文件夹
            if (folder.getInt(NoteColumns.TYPE) == Notes.TYPE_FOLDER) {
                // 获取文件夹的摘要作为名称，并添加前缀
                String name = folder.getString(NoteColumns.SNIPPET);
                setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX + name);
            }
            // 检查文件夹类型是否为系统文件夹
            else if (folder.getInt(NoteColumns.TYPE) == Notes.TYPE_SYSTEM) {
                // 根据系统文件夹的ID设置不同的名称
                if (folder.getLong(NoteColumns.ID) == Notes.ID_ROOT_FOLDER)
                    setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX + GTaskStringUtils.FOLDER_DEFAULT);
                else if (folder.getLong(NoteColumns.ID) == Notes.ID_CALL_RECORD_FOLDER)
                    setName(GTaskStringUtils.MIUI_FOLDER_PREFFIX
                            + GTaskStringUtils.FOLDER_CALL_NOTE);
                else
                    // 记录错误日志，提示无效的系统文件夹
                    Log.e(TAG, "invalid system folder");
            }
            else {
                // 记录错误日志，提示类型错误
                Log.e(TAG, "error type");
            }
        } catch (JSONException e) {
            // 记录错误日志并打印异常堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }
    }

    /**
     * 根据任务列表的内容生成本地JSON对象。
     *
     * @return 包含任务列表本地信息的JSON对象，若发生异常则返回null
     */
    public JSONObject getLocalJSONFromContent() {
        try {
            // 创建一个新的JSON对象
            JSONObject js = new JSONObject();
            // 创建文件夹信息的JSON对象
            JSONObject folder = new JSONObject();

            // 获取任务列表的名称
            String folderName = getName();
            // 如果名称以特定前缀开头，去除前缀
            if (getName().startsWith(GTaskStringUtils.MIUI_FOLDER_PREFFIX))
                folderName = folderName.substring(GTaskStringUtils.MIUI_FOLDER_PREFFIX.length(),
                        folderName.length());
            // 将文件夹名称添加到文件夹信息的JSON对象中
            folder.put(NoteColumns.SNIPPET, folderName);
            // 根据文件夹名称判断类型，并添加到文件夹信息的JSON对象中
            if (folderName.equals(GTaskStringUtils.FOLDER_DEFAULT)
                    || folderName.equals(GTaskStringUtils.FOLDER_CALL_NOTE))
                folder.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
            else
                folder.put(NoteColumns.TYPE, Notes.TYPE_FOLDER);

            // 将文件夹信息添加到JSON对象中
            js.put(GTaskStringUtils.META_HEAD_NOTE, folder);

            return js;
        } catch (JSONException e) {
            // 记录错误日志并打印异常堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 根据数据库游标判断任务列表的同步操作类型。
     *
     * @param c 数据库游标
     * @return 同步操作类型的常量值
     */
    public int getSyncAction(Cursor c) {
        try {
            // 检查本地是否有更新
            if (c.getInt(SqlNote.LOCAL_MODIFIED_COLUMN) == 0) {
                // 本地没有更新
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // 双方都没有更新
                    return SYNC_ACTION_NONE;
                } else {
                    // 应用远程更新到本地
                    return SYNC_ACTION_UPDATE_LOCAL;
                }
            } else {
                // 验证Google任务ID是否匹配
                if (!c.getString(SqlNote.GTASK_ID_COLUMN).equals(getGid())) {
                    // 记录错误日志，提示Google任务ID不匹配
                    Log.e(TAG, "gtask id doesn't match");
                    return SYNC_ACTION_ERROR;
                }
                if (c.getLong(SqlNote.SYNC_ID_COLUMN) == getLastModified()) {
                    // 只有本地有修改
                    return SYNC_ACTION_UPDATE_REMOTE;
                } else {
                    // 对于文件夹冲突，应用本地修改
                    return SYNC_ACTION_UPDATE_REMOTE;
                }
            }
        } catch (Exception e) {
            // 记录错误日志并打印异常堆栈信息
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }

        return SYNC_ACTION_ERROR;
    }

    /**
     * 获取任务列表中的子任务数量。
     *
     * @return 子任务的数量
     */
    public int getChildTaskCount() {
        return mChildren.size();
    }

    /**
     * 向任务列表中添加一个子任务。
     *
     * @param task 要添加的子任务
     * @return 添加成功返回true，失败返回false
     */
    public boolean addChildTask(Task task) {
        boolean ret = false;
        if (task != null && !mChildren.contains(task)) {
            // 将子任务添加到列表中
            ret = mChildren.add(task);
            if (ret) {
                // 设置子任务的前一个兄弟任务和父任务列表
                task.setPriorSibling(mChildren.isEmpty() ? null : mChildren
                        .get(mChildren.size() - 1));
                task.setParent(this);
            }
        }
        return ret;
    }

    /**
     * 在指定位置向任务列表中添加一个子任务。
     *
     * @param task 要添加的子任务
     * @param index 指定的位置
     * @return 添加成功返回true，失败返回false
     */
    public boolean addChildTask(Task task, int index) {
        if (index < 0 || index > mChildren.size()) {
            // 记录错误日志，提示索引无效
            Log.e(TAG, "add child task: invalid index");
            return false;
        }

        // 检查任务是否已存在于列表中
        int pos = mChildren.indexOf(task);
        if (task != null && pos == -1) {
            // 在指定位置插入子任务
            mChildren.add(index, task);

            // 更新任务列表中相关任务的前一个兄弟任务
            Task preTask = null;
            Task afterTask = null;
            if (index != 0)
                preTask = mChildren.get(index - 1);
            if (index != mChildren.size() - 1)
                afterTask = mChildren.get(index + 1);

            task.setPriorSibling(preTask);
            if (afterTask != null)
                afterTask.setPriorSibling(task);
        }

        return true;
    }

    /**
     * 从任务列表中移除一个子任务。
     *
     * @param task 要移除的子任务
     * @return 移除成功返回true，失败返回false
     */
    public boolean removeChildTask(Task task) {
        boolean ret = false;
        // 获取子任务在列表中的索引
        int index = mChildren.indexOf(task);
        if (index != -1) {
            // 从列表中移除子任务
            ret = mChildren.remove(task);

            if (ret) {
                // 重置子任务的前一个兄弟任务和父任务列表
                task.setPriorSibling(null);
                task.setParent(null);

                // 更新任务列表中相关任务的前一个兄弟任务
                if (index != mChildren.size()) {
                    mChildren.get(index).setPriorSibling(
                            index == 0 ? null : mChildren.get(index - 1));
                }
            }
        }
        return ret;
    }

    /**
     * 将一个子任务移动到指定位置。
     *
     * @param task 要移动的子任务
     * @param index 指定的位置
     * @return 移动成功返回true，失败返回false
     */
    public boolean moveChildTask(Task task, int index) {
        if (index < 0 || index >= mChildren.size()) {
            // 记录错误日志，提示索引无效
            Log.e(TAG, "move child task: invalid index");
            return false;
        }

        // 检查任务是否存在于列表中
        int pos = mChildren.indexOf(task);
        if (pos == -1) {
            // 记录错误日志，提示任务不在列表中
            Log.e(TAG, "move child task: the task should in the list");
            return false;
        }

        if (pos == index)
            return true;
        // 先移除任务，再在指定位置添加任务
        return (removeChildTask(task) && addChildTask(task, index));
    }

    /**
     * 根据全局ID查找任务列表中的子任务。
     *
     * @param gid 子任务的全局ID
     * @return 找到的子任务，若未找到则返回null
     */
    public Task findChildTaskByGid(String gid) {
        for (int i = 0; i < mChildren.size(); i++) {
            // 获取当前子任务
            Task t = mChildren.get(i);
            if (t.getGid().equals(gid)) {
                return t;
            }
        }
        return null;
    }

    /**
     * 获取指定子任务在任务列表中的索引。
     *
     * @param task 要查找索引的子任务
     * @return 子任务的索引，若未找到则返回-1
     */
    public int getChildTaskIndex(Task task) {
        return mChildren.indexOf(task);
    }

    /**
     * 根据索引获取任务列表中的子任务。
     *
     * @param index 子任务的索引
     * @return 找到的子任务，若索引无效则返回null
     */
    public Task getChildTaskByIndex(int index) {
        if (index < 0 || index >= mChildren.size()) {
            // 记录错误日志，提示索引无效
            Log.e(TAG, "getTaskByIndex: invalid index");
            return null;
        }
        return mChildren.get(index);
    }

    /**
     * 根据全局ID获取任务列表中的子任务。
     *
     * @param gid 子任务的全局ID
     * @return 找到的子任务，若未找到则返回null
     */
    public Task getChilTaskByGid(String gid) {
        for (Task task : mChildren) {
            if (task.getGid().equals(gid))
                return task;
        }
        return null;
    }

    /**
     * 获取任务列表中的所有子任务。
     *
     * @return 包含所有子任务的列表
     */
    public ArrayList<Task> getChildTaskList() {
        return this.mChildren;
    }

    /**
     * 设置任务列表在父级中的索引。
     *
     * @param index 要设置的索引
     */
    public void setIndex(int index) {
        this.mIndex = index;
    }

    /**
     * 获取任务列表在父级中的索引。
     *
     * @return 任务列表的索引
     */
    public int getIndex() {
        return this.mIndex;
    }
}