/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// 声明包名，该服务属于 net.micode.notes.gtask.remote 包
package net.micode.notes.gtask.remote;

// 导入 Activity 类，用于表示 Android 应用中的一个屏幕界面
import android.app.Activity;
// 导入 Service 类，这是该类的基类，用于实现后台服务
import android.app.Service;
// 导入 Context 类，用于提供应用程序的环境信息
import android.content.Context;
// 导入 Intent 类，用于在不同组件间传递消息和启动操作
import android.content.Intent;
// 导入 Bundle 类，用于在组件间传递数据
import android.os.Bundle;
// 导入 IBinder 类，用于在不同进程间进行通信
import android.os.IBinder;

/**
 * GTaskSyncService 类继承自 Service，用于处理 GTask 的同步服务。
 * 该服务提供了启动同步、取消同步等功能，并通过广播通知同步状态和进度。
 */
public class GTaskSyncService extends Service {
    // 定义一个常量字符串，用于在 Intent 中传递同步操作的类型
    public final static String ACTION_STRING_NAME = "sync_action_type";
    // 定义一个常量整数，表示启动同步的操作
    public final static int ACTION_START_SYNC = 0;
    // 定义一个常量整数，表示取消同步的操作
    public final static int ACTION_CANCEL_SYNC = 1;
    // 定义一个常量整数，表示无效的操作
    public final static int ACTION_INVALID = 2;
    // 定义一个常量字符串，作为同步服务广播的名称
    public final static String GTASK_SERVICE_BROADCAST_NAME = "net.micode.notes.gtask.remote.gtask_sync_service";
    // 定义一个常量字符串，用于在广播中表示是否正在同步
    public final static String GTASK_SERVICE_BROADCAST_IS_SYNCING = "isSyncing";
    // 定义一个常量字符串，用于在广播中传递同步进度消息
    public final static String GTASK_SERVICE_BROADCAST_PROGRESS_MSG = "progressMsg";

    // 静态变量，用于存储当前的同步任务
    private static GTaskASyncTask mSyncTask = null;
    // 静态变量，用于存储同步进度消息
    private static String mSyncProgress = "";

    /**
     * 启动同步任务的方法。
     * 如果当前没有正在进行的同步任务，则创建一个新的同步任务并执行。
     */
    private void startSync() {
        // 检查当前是否没有同步任务正在进行
        if (mSyncTask == null) {
            // 创建一个新的 GTaskASyncTask 实例，并传入上下文和完成监听器
            mSyncTask = new GTaskASyncTask(this, new GTaskASyncTask.OnCompleteListener() {
                /**
                 * 同步任务完成时调用的方法。
                 * 重置同步任务，发送广播通知同步完成，并停止服务。
                 */
                public void onComplete() {
                    // 同步任务完成后，将 mSyncTask 置为 null
                    mSyncTask = null;
                    // 发送广播，通知同步完成，传入空消息
                    sendBroadcast("");
                    // 停止当前服务
                    stopSelf();
                }
            });
            // 发送广播，通知同步开始，传入空消息
            sendBroadcast("");
            // 执行同步任务
            mSyncTask.execute();
        }
    }

    /**
     * 取消同步任务的方法。
     * 如果当前有正在进行的同步任务，则取消该任务。
     */
    private void cancelSync() {
        // 检查当前是否有同步任务正在进行
        if (mSyncTask != null) {
            // 调用同步任务的 cancelSync 方法取消同步
            mSyncTask.cancelSync();
        }
    }

    /**
     * 服务创建时调用的方法。
     * 初始化同步任务为 null。
     */
    @Override
    public void onCreate() {
        // 服务创建时，将 mSyncTask 置为 null
        mSyncTask = null;
    }

    /**
     * 服务启动时调用的方法。
     * 根据传入的 Intent 中的操作类型，执行相应的同步操作。
     *
     * @param intent  启动服务时传入的 Intent
     * @param flags   启动标志
     * @param startId 启动 ID
     * @return 返回服务的启动模式
     */
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        // 从 Intent 中获取 Bundle 数据
        Bundle bundle = intent.getExtras();
        // 检查 Bundle 是否存在且包含同步操作类型
        if (bundle != null && bundle.containsKey(ACTION_STRING_NAME)) {
            // 根据 Bundle 中的操作类型进行不同处理
            switch (bundle.getInt(ACTION_STRING_NAME, ACTION_INVALID)) {
                case ACTION_START_SYNC:
                    // 启动同步任务
                    startSync();
                    break;
                case ACTION_CANCEL_SYNC:
                    // 取消同步任务
                    cancelSync();
                    break;
                default:
                    break;
            }
            // 返回 START_STICKY 表示服务在被系统杀死后会自动重启
            return START_STICKY;
        }
        // 如果 Bundle 不包含有效操作类型，调用父类的 onStartCommand 方法
        return super.onStartCommand(intent, flags, startId);
    }

    /**
     * 系统内存不足时调用的方法。
     * 如果当前有正在进行的同步任务，则取消该任务。
     */
    @Override
    public void onLowMemory() {
        // 检查当前是否有同步任务正在进行
        if (mSyncTask != null) {
            // 调用同步任务的 cancelSync 方法取消同步
            mSyncTask.cancelSync();
        }
    }

    /**
     * 绑定服务时调用的方法。
     * 由于该服务不需要绑定，返回 null。
     *
     * @param intent 绑定服务时传入的 Intent
     * @return 返回 null
     */
    public IBinder onBind(Intent intent) {
        return null;
    }

    /**
     * 发送广播通知同步状态和进度的方法。
     *
     * @param msg 同步进度消息
     */
    public void sendBroadcast(String msg) {
        // 更新同步进度消息
        mSyncProgress = msg;
        // 创建一个新的 Intent，用于发送广播
        Intent intent = new Intent(GTASK_SERVICE_BROADCAST_NAME);
        // 在 Intent 中添加是否正在同步的信息
        intent.putExtra(GTASK_SERVICE_BROADCAST_IS_SYNCING, mSyncTask != null);
        // 在 Intent 中添加同步进度消息
        intent.putExtra(GTASK_SERVICE_BROADCAST_PROGRESS_MSG, msg);
        // 发送广播
        sendBroadcast(intent);
    }

    /**
     * 静态方法，用于从 Activity 启动同步服务。
     *
     * @param activity 启动同步服务的 Activity
     */
    public static void startSync(Activity activity) {
        // 设置 GTaskManager 的 Activity 上下文
        GTaskManager.getInstance().setActivityContext(activity);
        // 创建一个新的 Intent，用于启动 GTaskSyncService
        Intent intent = new Intent(activity, GTaskSyncService.class);
        // 在 Intent 中添加启动同步的操作类型
        intent.putExtra(GTaskSyncService.ACTION_STRING_NAME, GTaskSyncService.ACTION_START_SYNC);
        // 启动服务
        activity.startService(intent);
    }

    /**
     * 静态方法，用于取消同步服务。
     *
     * @param context 上下文对象
     */
    public static void cancelSync(Context context) {
        // 创建一个新的 Intent，用于启动 GTaskSyncService
        Intent intent = new Intent(context, GTaskSyncService.class);
        // 在 Intent 中添加取消同步的操作类型
        intent.putExtra(GTaskSyncService.ACTION_STRING_NAME, GTaskSyncService.ACTION_CANCEL_SYNC);
        // 启动服务
        context.startService(intent);
    }

    /**
     * 静态方法，用于检查当前是否正在进行同步。
     *
     * @return 如果正在同步返回 true，否则返回 false
     */
    public static boolean isSyncing() {
        // 判断 mSyncTask 是否不为 null，即是否有同步任务正在进行
        return mSyncTask != null;
    }

    /**
     * 静态方法，用于获取当前的同步进度消息。
     *
     * @return 当前的同步进度消息
     */
    public static String getProgressString() {
        // 返回当前的同步进度消息
        return mSyncProgress;
    }
}